package ca.ualberta.cs.poker.free.server;

import com.biotools.meerkat.*;
import com.biotools.meerkat.util.*;
import java.io.*;
import java.net.*;
import javax.swing.*;
import ca.ualberta.cs.poker.free.server.*;

 /*
  * @author Martin Zinkevich
  */
public class PAProTCPPlugin implements Player{
    private ServerSocket serverSocket = null;
    private Card holeCard1 = null;
    private Card holeCard2 = null;
    private Card oppCard1 = null;
    private Card oppCard2 = null;
    private int uofagameIndex = 0;
    private StringBuffer bettingSequence = new StringBuffer();
    private static final int  DEFAULT_SERVER_PORT = 5000;
    private int paseat = 0;
    private int uoaseat = -1;
    private GameInfo gameInfo = null;
    private int serverPort = 0;
    private JFrame frame = null;
    private JTextArea textArea = null;
    private boolean debug = true;
    private static final int MAX_LINES = 50;
    private static final String PROTOCOL_VERSION = "VERSION:1.0.0";
    private PlayerAgent playerAgent = null;
    private static final int EXIT_FAILURE = -1;
    private static final int UOA_BUTTON_SEAT=0;
    private static final int UOA_NON_BUTTON_SEAT = 1;
    int lineCount = 0;
    private static final String ENDGAME_HEADER = "ENDGAME";
    
    /** Creates a new instance of TCPPlugin */
    public PAProTCPPlugin() {
        initDebugFrame();
        log("<poker academy interface instantiated>", null);
        uofagameIndex = 0;
    }
    
    public void init(Preferences prefs){
        /// Waits for client at init state
        
        /// Get the requested port number from the preference file.
        serverPort = prefs.getIntPreference("PORT", DEFAULT_SERVER_PORT); try {
        serverSocket = new ServerSocket(serverPort);
        } catch (IOException io) {
            log("<init> failed to bind server socket on port" + serverPort +"?", io);
        }
        
        waitForClient();
    }
    
    public com.biotools.meerkat.Action getAction(){
        log("<get action>", null);
        /// MATCHSTATE should be consistent with what server expects (last sent state to player)
        /// SEND MATCHSTATE:SEAT_ID:GAME_ID:GAME_STATE:ACTION
        try {
            com.biotools.meerkat.Action a = receiveAction();
            log("received action " + a + " from client" , null);
            return a;
            //} catch (TimeoutException to) {
            //  log("timeout waiting for action from player", to);
        } catch (SyncException se) {
            log("protocol sync error", se);
        } catch (IOException e) {
            log("io error. assuming game is over", e);
            System.exit(0);
        }
        return com.biotools.meerkat.Action.sitout();
    }
    
    private char actionToChar(com.biotools.meerkat.Action act) throws ConversionException {
        if (act.isCheckOrCall()) {
            return 'c';
        } else if (act.isBetOrRaise()) {
            return 'r';
        } else if (act.isFold()) {
            return 'f';
        } else {
            throw new ConversionException(act);
        }
        
    }
    
    public void stageEvent(int pos) {
        log("<stage event>", null);
    }
    
    public void actionEvent(int pos, com.biotools.meerkat.Action act) {
        log( "<action event>", null);
        if ((act.isSmallBlind()) || (act.isBigBlind()))
                return;
        
        /// Update bettingSequence buffer
        try {
            bettingSequence.append(actionToChar(act));
        } catch (ConversionException e) {
            log("<conversion exception, act = " + e.getAction() + "> ", e);            
            return;
        }
        
        try {
            /// Send action event to player
            playerAgent.sendMessage(getMatchState());
        } catch (Exception e) {
            log("exception ocurred in actionEvent. ", e);
            System.exit(EXIT_FAILURE);
        }
    }
    
    public void gameStartEvent(GameInfo info){
        log( "<game start event>", null);
        uofagameIndex++;
        bettingSequence = new StringBuffer();
        this.gameInfo = info;
    }
    
    public void gameStateChanged(){
        log( "<game state changed- ignoring>", null);
    }
    
    public void holeCards(Card h1, Card h2, int seat){
        
        holeCard1 = h1;
        holeCard2 = h2;
        paseat = seat;
        /// The seat in this protocol is always determined by who has the button
        if (paseat == gameInfo.getButtonSeat())
            uoaseat = UOA_BUTTON_SEAT;
        else
            uoaseat = UOA_NON_BUTTON_SEAT;
        
        log( "<hole cards received by PAPro: " + h1 + h2 + ", paseat = " + paseat +", " + " uoaseat = " + uoaseat + ">", null);
        /// SERVER SENDS: MATCHSTATE:PLAYER_SEAT:GAME_ID::HOLE_CARDS| to this player's seat in the uoa protocol version (might keep the same as PA)
        try {
            playerAgent.sendMessage(getMatchState());
        } catch (TimeoutException to) {
            log("<timeout occurred>", to);
        }
    }
    
    
    public void dealHoleCardsEvent(){
        log( "<hole cards event>", null);
    }
    
    public void winEvent(int seat, double amount, String cards){
        log("<win event>", null);
    }
    
    public void gameOverEvent(){
        log("<game over event>", null);
    }
    
    public void showdownEvent(int seat, Card c1, Card c2){
        log("<showdown event>", null);
        if (seat!=paseat){
            oppCard1 = c1;
            oppCard2 = c2;
        }
        try {
            playerAgent.sendMessage(getMatchState());
        } catch (TimeoutException to) {
            log("<timeout occurred@showdown>", to);
        }
    }
    public com.biotools.meerkat.Action receiveAction() throws IOException, SyncException {
        String response = null;
        do{
            try {
            response = playerAgent.receiveMessage();
            } catch (TimeoutException to) {
                log("<timeout occurred@receiveAction>", to);
            }            
        } while(!isAppropriate(response)); /// why is this repeated?
        
        char c = getActionFromResponse(response);
        
        switch (c) {
            case 'f':
                return com.biotools.meerkat.Action.foldAction(gameInfo);
            case 'c':
                return com.biotools.meerkat.Action.callAction(gameInfo);
            case 'r':
                return com.biotools.meerkat.Action.betAction(gameInfo);
            default:
                throw new SyncException();
                
        }
        
    }
    
    /**
     * Tests if a response is actually a response to the CURRENT action.
     */
    public boolean isAppropriate(String response){
        if (response.length()<2){
            return false;
        }
        
        return true;
        /// Ignoring this code for now until I check the logic myself
        //String responseFront = response.substring(0,response.length()-2);
        //System.out.println("FRONT OF RESPONSE:"+responseFront);
        //System.out.println("MATCH STATE:"+getMatchState(seatToAct));
        //return (getMatchState(seatToAct).equals(responseFront));
        
    }
    
    /**
     * Gets the last character of a response, which should be 'c', 'r', or 'f'
     */
    public char getActionFromResponse(String response){
        return response.charAt(response.length()-1);
    }
    
    public void sendMatchState()throws TimeoutException {
        playerAgent.sendMessage(getMatchState());
    }
    
    public String getMatchState() {
        String result = "MATCHSTATE:" + uoaseat + ":" + uofagameIndex + ":"+bettingSequence +":";
        String oppCards = (oppCard1==null) ? "" : ("" + oppCard1 + oppCard2);
        String holeCards = (holeCard1==null) ? "" : ("" + holeCard1 + holeCard2);
        String nonButtonCards = (paseat==gameInfo.getButtonSeat()) ? holeCards : oppCards;
        String buttonCards = (paseat==gameInfo.getButtonSeat()) ? oppCards : holeCards;
        result = result + nonButtonCards + "|" + buttonCards;
        Hand board = gameInfo.getBoard();
        if (board.size()>=3){
            String flop = "/"+ board.getCard(1) + board.getCard(2) + board.getCard(3);
            result = result + flop;
        }
        if (board.size()>=4){
            String turn = "/" + board.getCard(4);
            result = result + turn;
        }
        if (board.size()>=5){
            String river = "/" + board.getCard(5);
            result = result + river;
        }
        
        return result;
    }
    
    private void initializeServer() {
        try{
            serverSocket = new ServerSocket(serverPort);
        }catch (IOException io){
            log("cannot start server. specified port is " + serverPort + "(0 == any)", io);
            return;
        }
        /// Blocks until a client is authenticated
        waitForClient();
        
    }
    
    private void handleEndGame() {
        try{
            playerAgent.sendMessage(ENDGAME_HEADER);
            playerAgent.close();
            
            serverSocket.close();
        } catch (TimeoutException e){
            log("Timeout exception at endgame.", e);
        } catch (IOException io){
            log("IOException at endgame.", io);
        }
        //log("Elapsed sending time for player " + i + ":"+playerAgent.elapsedSendingTime, null);
        //log("Final bankroll["+i+"]="+playerAgent.bankroll, null);
    }
    
    private void initDebugFrame() {
        
        frame = new JFrame("UAlberta Debug Console");
        textArea = new JTextArea();
        
        JScrollPane scrollPane = new JScrollPane(textArea);
        
        frame.add(scrollPane);
        frame.pack();
        frame.setVisible(true);
        
    }
    
    
    private void log(String message, Exception e) {
        
        if (debug) {
            ByteArrayOutputStream byteBuffer = new ByteArrayOutputStream();
            PrintStream printStream = new PrintStream(byteBuffer);
            if (e != null)
                e.printStackTrace(printStream);
            
            textArea.setText(textArea.getText() + message + "\n" + byteBuffer.toString());
            lineCount++;
        }
    }
    
    public void waitForClient() {
        try{
            Socket childSocket = serverSocket.accept();
            InetAddress applicantAddress = childSocket.getInetAddress();
            try {
                playerAgent=new PlayerAgent(childSocket,0);
            } catch (SocketException so) {
                log("connection appears broken. port = " + serverPort, so);
            } catch (IOException io){
                log("connection appears broken. port = " + serverPort, io);
            }
            
            log("SERVER ACCEPTED CLIENT FROM " + childSocket.getInetAddress(), null);
            
        } catch (IOException io){
            log("CRITICAL ERROR: CANNOT START SERVER. port =" +serverPort, io);
            return;
        }
        
        
        playerAgent.setTimeRemaining(1000);
        
        try{
            String message = playerAgent.receiveMessage();
            if (!message.equals(PROTOCOL_VERSION)) {
                log("player client does not acknowledge the protocol (version header should be \"" + PROTOCOL_VERSION + "\", instead got \"" + message + "\")", null);
            }
        } catch(TimeoutException to){
            log("timeout waiting for player to acknoledge the protocol", to);
        }
        
    }
}

