package ca.ualberta.cs.poker.free.tournament;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedList;

public class Forge {
	
	private LinkedList l = new LinkedList();
	private ArrayList availIPs;
	
	public Forge () {
		availIPs = new ArrayList();
		//ips = new ArrayList();
	}
	
	/*
	 * Add a match to the list
	 */
	public void add( Match m ) {
		l.addLast( m );
		
		//System.out.println( "Added " + m.getComp1() + " vs " + m.getComp2() + " match " + m.getMatch());
	}
	
	/*
	 * The forge needs to keep track of which IPs are free, this is
	 * done at initialization
	 */
	public void addIPToAvailable( String ip ) {
		// dont add duplicates, sometimes errors will make it try to be added twice
		if ( !availIPs.contains( ip )) {
			availIPs.add( ip );
		}
	}
	
	/*
	 * When freeing up an IP because a match is over, called internally
	 */
	private void removeIPFromAvailable( String ip ) {
		for ( int i = 0; i < availIPs.size(); i++ ) {
			if ( ((String) availIPs.get(i)).equals( ip ) ) {
				availIPs.remove( i );
			}
		}
		
	}
	
	/*
	 * If 2 IPs, and at least one match are present, we're ready to go
	 */
	public boolean isReady() {
		if( (availIPs.size() >= 2) && (l.size() > 0)){
			return true; 
		}
		
		return false;
	}
	
	/*
	 * Called by series when a match is done. 
	 * Should free up the IPs
	 */
	public void completeMatch( Match m ) {
		
		// if there are previous results, IP will be null, don't add 
		
//		if ( m.getResultFile() != null && m.getIp1() != null && m.getIp2() != null) {
//			
//			// Append some log data to the result file for later verification
//	        BufferedWriter out;
//			try {
//				out = new BufferedWriter(new FileWriter(m.getResultFile(), true));
//				out.write("\nForge: Comp1 = " + m.getComp1() + " Comp2 = " + m.getComp2());
//				out.write( "\nForge: IPs " + m.getIp1().toString() + " " + m.getIp2().toString() );
//		        out.close();
//		        
//			} catch (IOException e) {
//				// TODO Auto-generated catch block
//				e.printStackTrace();
//			}
//		}
	
		if (m.getIp1() != null ) {
			addIPToAvailable( m.getIp1() );
		}
		
		if (m.getIp2() != null ) {
			
			addIPToAvailable( m.getIp2() );
		}
		
		l.remove( (Match) m );
		
	}
	
	
	
	/*
	 * Remove the match from the list once completed
	 */
//	public void remove( Match m ) {
//		
//		for ( int i = 0; i < l.size(); i ++ ) {
//			
//				if ( ((Match)l.get( i )).equals( m )  ) {
//					l.remove( i );
//				}
//		}
//	}
	
	
	public Match getFirstMatch() { 
		
		Match m = (Match) l.getFirst();
		String ip1 = (String) availIPs.get(0);
		String ip2 = (String) availIPs.get(1);
		
		availIPs.remove( 0 );
		availIPs.remove( 0 );
		
		m.setIp1( ip1 );
		m.setIp2( ip2 );
		
		if ( l.size() > 0 ) {
			return (Match)l.removeFirst();
		}
		
		return null; 
	}
	
	// search the forge for the match in question
	public boolean hasMatch( Match m ) {
		
		boolean found = false;
		
		for ( int i = 0; i < l.size(); i ++ ) {
			
			if ( ((Match)l.get( i )).equals( m )  ) {
				found = true;
			}
		}
		
		return found;	
	}
	
	// search the forge for the match in question
	public boolean hasMatch( int series, int match ) {		
		
		boolean found = false;
		
		for ( int i = 0; i < l.size(); i ++ ) {
			
			if ( (((Match)l.get( i )).getSeries() == series) && (((Match)l.get( i )).getMatch() == match)  ) {
				found = true;
			}
		}
		
		return found;	
	}
	
	public boolean hasMatch() {
		
		if ( l.size() > 0 ) return true;
		else return false;
	}
	
	public int hasNumberOfMatches() {
		return l.size();
	}
	
	public int hasNumberOfIPs() {
		return availIPs.size();
	}
	
	
	public String toString() {
		String ret = "Forge: ";
		ret += "size = " + l.size() + " : ";
		
		for ( int i = 0; i < l.size(); i ++ ) {
			ret += ((Match)l.get( i )).toString();
		}
		
		return ret;
	}
	
}