/**
 * 
 */
package ca.ualberta.cs.poker.free.tournament;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;

/**
 * @author Christian Smith
 *
 */
public class RoundRobinSeries implements Node {

	private boolean isComplete;
	private int numCompetitors;
	private int numMatches;
	private boolean inForge;
	private ArrayList allMatches;
	private double[] numWins;
	private double[] bankrollTotals;
	private int timePerHand;
	
	public RoundRobinSeries( int _numCompetitors, int _numMatches, int _timePerHand ) {
		
		numCompetitors = _numCompetitors;
		numMatches = _numMatches;
		timePerHand = _timePerHand;
		inForge = false;
		isComplete = false;
		allMatches = new ArrayList();
		GenerateMatches();
		numWins = new double[numCompetitors];
		bankrollTotals = new double[numCompetitors];
	}
	
	/* 
	 * 
	 */
	public boolean complete() {
		return isComplete;
	}

	/* 
	 * Load the appropriate forge full of matches needed to be run
	 * 
	 * In RoundRobin this end up simply being all matches that need run.
	 * Load was more intuitive in Single Elimination where the matches
	 * needed to be run depending on the order of wins
	 * 
	 */
	public boolean load(Forge f) {
		
		if ( inForge == false ) {
			
			ArrayList matches = getMatches();
			
			for ( int i = 0; i < matches.size(); i++ ) {
				f.add( (Match) matches.get( i ) );	
			}										
			
			inForge = true;
		}
		
		
			
		// otherwise we are in the forge, check to see if we're done
		boolean allMatchesCompleted = true;
		ArrayList matches = getMatches();
		
		for ( int i = 0; i < matches.size(); i++ ) {
			Match m = (Match)matches.get(i);
			
			// Check on disk
			
			// ie. data/results/seseries3match2.res
			boolean exists = (new File(m.getResultFile())).exists();
			//System.out.println ( "Exists?" + m.getResultFile() );
			
			// if the .res file is there, the server has finished that match
			if ( exists ) {
				f.completeMatch( m );
			}
			else {
				allMatchesCompleted = false;
			}
			
		}
		
		
		if ( allMatchesCompleted ) {
			// determine winner
			
			double bankrollForPlayer1 = 0.0;	
			double bankrollForPlayer2 = 0.0;
			numWins = new double[numCompetitors + 1];
			bankrollTotals = new double[numCompetitors + 1];
			
			
			for ( int i = 0; i < matches.size(); i++ ) {
					
				Match m = (Match)matches.get(i);
				// read the bankroll from the .res file
				try {
					BufferedReader in = new BufferedReader(new FileReader(((Match)matches.get(i)).getResultFile()));
			        String str;
			        
			        try {
						
						str = in.readLine();
						in.close();
						
						// first, increment number of wins
						bankrollForPlayer1 = Double.parseDouble(str);
						bankrollForPlayer2 = Double.parseDouble(str) * -1;
						
						
						if ( bankrollForPlayer1 > 0 ) {
							numWins[ m.getCompNum1() ] = numWins[ m.getCompNum1() ] + 1;
						}
						if ( bankrollForPlayer1 < 0 ) {
							numWins[ m.getCompNum2() ] = numWins[ m.getCompNum2() ] + 1;
						}
						// if tie rules are necessary
						
						if ( bankrollForPlayer1 == 0 ) {
							numWins[ m.getCompNum1() ] = numWins[ m.getCompNum1() ] + 0.5;
							numWins[ m.getCompNum2() ] = numWins[ m.getCompNum2() ] + 0.5;
						}
						
						// now add the bankrolls to total
						bankrollTotals[ m.getCompNum1() ] += bankrollForPlayer1;
						bankrollTotals[ m.getCompNum2() ] += bankrollForPlayer2;
						
						
						//System.out.println( "Found " + bankrollForPlayer1 );
						System.out.println( "Wins now ");
						for ( int j = 1; j <= numCompetitors; j++ ) {
							System.out.print( numWins[j] + " " );		
						}
						
						
						System.out.println( "\nBankroll totals now "); 
						for ( int j = 1; j <= numCompetitors; j++ ) {
							System.out.print( bankrollTotals[j] );		
						}		
						System.out.println( "\n");		
						                              
						
					} catch (IOException e) {
						System.err.println( "Error reading double from result file");
						e.printStackTrace();
					}
			        
					
					
				} catch (FileNotFoundException e) {
					System.err.println( "Missing result file " + ((Match)matches.get(i)).getResultFile());
					e.printStackTrace();
				}
				
				
				
			}
			
			
			// no longer in forge
			inForge = false;
			isComplete = true;
			
		} // end if allmatchescompleted
		
		
		
		
		
		
		
		
		return isComplete;
	}

	
	/**
	 * Generate an arraylist of the matches that need to be run in a
	 * round-robin tournament
	 * 
	 * @return Arraylist of all the matches needed to be run by the tournament
	 */
	public void GenerateMatches() {
		
		for (int i = 1; i <= numCompetitors; i++) {
			for (int j = i + 1; j <= numCompetitors; j++) {
				for (int k = 1; k <= numMatches; k++) {
					
					// use the constructor with ints, so we can more simply do
					// operations on the competitor number later
					Match m = new Match( i, j, null, null, 1, k, "RR", timePerHand);
					allMatches.add( (Match) m );			
					System.out.println( "Adding " + "comp" + i + " " + "comp" + j + " " + k);
					
					m = new Match( j, i, null, null, 1, k, "RR", timePerHand );
					m.flipRRFilePaths();
					
					
					allMatches.add( (Match) m );			
					System.out.println( "Adding " + "comp" + j + " " + "comp" + i + " " + k);
					
					
				}
			}
		}
	}
	
	public ArrayList getMatches() {
		return allMatches;
	}
	
	/* 
	 * Determine the winner with the array of winning "points"
	 */
	public String getWinner() {

		if ( isComplete ) {

			int indexOfWinner = 0;
			double highest = 0.0;
			
			for ( int i = 0; i < numCompetitors; i ++ ) {
				
				if ( numWins[ i ] > highest  ) {
					highest = numWins[ i ];
					indexOfWinner = i;
				}
				
			}
			
			return "Competitor #" + (indexOfWinner); // correct for indexing from 0
			
		}
		
		return null;
	}

}
