package ca.ualberta.cs.poker.free.tournament;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Random;

public class Series implements Node {
	
	private Node left;
	private Node right;
	private boolean isComplete;
	private boolean inForge;
	private int num;
	private int numMatches;
	private int randomWinner;
	private String winner;
	private ArrayList matches;
	private boolean allMatchesCompleted;
	
	public Series ( int _num, int _numMatches, Node comp1, Node comp2 ) {
		left = comp1;
		right = comp2;
		isComplete = false;
		inForge = false;
		num = _num;
		numMatches = _numMatches;
		matches = new ArrayList();
		winner = "";
		
		// we need to have a random winner in case of a bankroll tie
		Random rand = new Random();
		randomWinner = rand.nextInt(2);
		
	}
	
	public boolean complete() {
		
		//if ( left.complete() && right.complete() ) {
		//	return true;
		//}
		
		return isComplete;
	}
	
	/*
	 * Recursively load any matches that need to be run into the 'forge'
	 *
	 * Node in the tree can be in three states: 
	 *
	 * complete, !complete and inForge, !complete and !inForge
	 *
	 * Complete nodes are finished and getWinner() will reveal the winner of all the
	 * matches. If its matches are in the forge, every call to load should check for
	 * the existance of the .res files which will be written when the match is completed
	 * successfully. Once load finds all the proper .res files, the series is complete
	 * and the winner should be determined for future reference.
	 *
	 */
	public boolean load (Forge f) {
		
		//System.out.println( "Loading Series " + num );
		
		if ( isComplete ) return true;
		
		// isComplete is false, so either load this series or those below
		if ( left.complete() == false ) {
			left.load( f );
		}
		
		if ( right.complete() == false ) {
			right.load( f );
		}
		
		// if no in the forge, and not complete, load the forge with your matches
		if ( inForge == false ) {
			
			// dont try to load a series that isn't ready yet
			if ( (left.getWinner() != null) && (right.getWinner() != null ) ){	
					
				ArrayList matches = generateMatches();
					for ( int i = 0; i < matches.size(); i ++ ) {
						f.add( (Match)matches.get(i) );
					}
					
					// matches are in the forge now
					inForge = true;
				
			}
			
			
		}
		// if we are in forge, see if any of the matches have completed
		if ( inForge == true ) {
			// check existing on disk, remove finished from Forge
			ArrayList al = getMatches();
			
			allMatchesCompleted = true;
			
			for ( int i = 0; i < al.size(); i++ ) {
				Match m = (Match)al.get(i);
				//if ( f.hasMatch(m) ){
					
					// Check on disk
					
					// ie. data/results/seseries3match2.res
					boolean exists = (new File(m.getResultFile())).exists();
					//System.out.println ( "Exists?" + m.getResultFile() );
					
					// if the .res file is there, the server has finished that match
					if ( exists ) {
						f.completeMatch( m );
						
					}
					else {
						allMatchesCompleted = false;
					}
				
			}
		
			if ( allMatchesCompleted ) {
				// determine winner
				
				double runningTotalForPlayer0 = 0.0;
				ArrayList matches = getMatches();
				
				
				for ( int i = 0; i < matches.size(); i++ ) {
						
					// read the bankroll from the .res file
					try {
						BufferedReader in = new BufferedReader(new FileReader(((Match)matches.get(i)).getResultFile()));
				        String str;
				        
				        try {
							
							str = in.readLine();
							in.close();
							runningTotalForPlayer0 += Double.parseDouble(str);
							
						} catch (IOException e) {
							System.err.println( "Error reading double from result file");
							e.printStackTrace();
						}
				        
						
						
					} catch (FileNotFoundException e) {
						System.err.println( "Missing result file " + ((Match)matches.get(i)).getResultFile());
						e.printStackTrace();
					}
					
					
					
				}
				
				if ( runningTotalForPlayer0 > 0 ) {
					winner = left.getWinner();
				}
				if ( runningTotalForPlayer0 < 0 ){
					winner = right.getWinner();
				}
				
				// we have a tie, determine winner randomly
				if ( runningTotalForPlayer0 == 0 && randomWinner == 0 ) {
					
						winner = left.getWinner();
				}
				if ( runningTotalForPlayer0 == 0 && randomWinner == 1 ) {
					
					winner = right.getWinner();
				}
				
				
				// no longer in forge
				inForge = false;
				isComplete = true;
				
			}
			
		}
			
		
		
		return isComplete;
	}
	
	
	/*
	 * Helper function for querying the forge to see if any particular match
	 * is in the forge
	 */
	public boolean hasMatchInForge( Forge f ) {
		
		ArrayList matches = getMatches();
		boolean found = false;
		
		for ( int i = 0; i < matches.size(); i ++ ) {
			if ( f.hasMatch( (Match) matches.get(i) ) ) {
				found = true;
			}
		}
		
		return found;
	}
	
	public ArrayList getMatches() {
		return matches;
		
	}
	
	/*
	 * Get the matches associated with this series
	 */
	public ArrayList generateMatches() {
		
		String leftWinner = left.getWinner();
		String rightWinner = right.getWinner();
		
		if ( (leftWinner != null)  && (rightWinner != null) ) {
			
			for ( int i = 1; i <= numMatches; i++ ) {
				Match m = new Match(leftWinner, rightWinner, null, null, num, i, "SE", 7000);
				matches.add( m );
			}
		}
		
		return matches;
	}
	
	/*
	 * Get the winner associated with this series, null if not determined yet
	 */
	public String getWinner() {
		
		if ( isComplete ) {
			//System.out.println( "Series" + num + " winner = " + winner );
			return winner;
		}
		else
			return null;
	}
	
	public String toString () {
		return "Series " + num + "r: " + right.toString() + "  l: " + left.toString();
	}
	
}